@@ -81,7 +81,7 @@ headtofb(struct rpmhead *h, struct fileblock *fb)
     }
   fb->filemodes = headint16(h, TAG_FILEMODES, (int *)0);
   fb->filesizes = headint32(h, TAG_FILESIZES, (int *)0);
-  fb->filerdevs = headint32(h, TAG_FILERDEVS, (int *)0);
+  fb->filerdevs = headint16(h, TAG_FILERDEVS, (int *)0);
   fb->filelinktos = headstringarray(h, TAG_FILELINKTOS, (int *)0);
   fb->filemd5s = headstringarray(h, TAG_FILEMD5S, (int *)0);
   return 0;
@@ -120,7 +120,7 @@ checkfilemd5(char *name, unsigned char *hmd5, int size)
     {
       if (l > size)
 	l = size;
-      rpmMD5Update(&ctx, buf, l);
+      rpmMD5Update(&ctx, (unsigned char *)buf, l);
       size -= l;
     }
   close(fd);
@@ -153,7 +153,8 @@ checkfilesize(char *name, int size)
 struct seqdescr *
 expandseq(unsigned char *seq, int seql, int *nump, struct fileblock *fb, int flags)
 {
-  unsigned char *s, *fn;
+  unsigned char *s;
+  char *fn;
   int *res;
   int i, n, n2, num, nib, shi, tog, jump, pos;
   unsigned int rdev, lsize;
@@ -246,7 +247,7 @@ expandseq(unsigned char *seq, int seql, int *nump, struct fileblock *fb, int fla
       fn = fb->filenames[i];
       if (*fn == '/')
 	fn++;
-      rpmMD5Update(&seqmd5, fn, strlen(fn) + 1);
+      rpmMD5Update(&seqmd5, (unsigned char *)fn, strlen(fn) + 1);
       rpmMD5Update32(&seqmd5, fb->filemodes[i]);
       rpmMD5Update32(&seqmd5, lsize);
       rpmMD5Update32(&seqmd5, rdev);
@@ -257,7 +258,7 @@ expandseq(unsigned char *seq, int seql, int *nump, struct fileblock *fb, int fla
       if (sd[n].datalen & 3)
 	sd[n].datalen += 4 - (sd[n].datalen & 3);
       if (S_ISLNK(fb->filemodes[i]))
-	rpmMD5Update(&seqmd5, fb->filelinktos[i], strlen(fb->filelinktos[i]) + 1);
+	rpmMD5Update(&seqmd5, (unsigned char *)fb->filelinktos[i], strlen(fb->filelinktos[i]) + 1);
       else if (S_ISREG(fb->filemodes[i]) && lsize)
 	{
 	  parsemd5(fb->filemd5s[i], fmd5);
@@ -519,7 +520,7 @@ createcpiohead(struct seqdescr *sd, struct fileblock *fb)
 
   if (i == -1)
     {
-      sprintf(cpiodata, "%s%c%c%c%c", "07070100000000000000000000000000000000000000010000000000000000000000000000000000000000000000000000000b00000000TRAILER!!!", 0, 0, 0, 0);
+      sprintf((char *)cpiodata, "%s%c%c%c%c", "07070100000000000000000000000000000000000000010000000000000000000000000000000000000000000000000000000b00000000TRAILER!!!", 0, 0, 0, 0);
       return;
     }
   lsize = rdev = 0;
@@ -535,7 +536,7 @@ createcpiohead(struct seqdescr *sd, struct fileblock *fb)
     }
   if (S_ISBLK(fb->filemodes[i]) || S_ISCHR(fb->filemodes[i]))
     rdev = fb->filerdevs[i];
-  sprintf(cpiodata, "07070100000000%08x00000000000000000000000100000000%08x0000000000000000%08x%08x%08x00000000./%s%c%c%c%c", fb->filemodes[i], lsize, devmajor(rdev), devminor(rdev), (int)strlen(np) + 3, np, 0, 0, 0, 0);
+  sprintf((char *)cpiodata, "07070100000000%08x00000000000000000000000100000000%08x0000000000000000%08x%08x%08x00000000./%s%c%c%c%c", fb->filemodes[i], lsize, devmajor(rdev), devminor(rdev), (int)strlen(np) + 3, np, 0, 0, 0, 0);
 }
 
 void
@@ -598,7 +599,7 @@ fillblock_disk(struct blk *b, int id, struct seqdescr *sdesc, int nsdesc, struct
 	      if (u > strlen(symdata))
 		memset(bp, 0, l2);
 	      else
-		strncpy(bp, symdata + u, l2);
+		strncpy((char *)bp, symdata + u, l2);
 	    }
 	  else if (u < fb->filesizes[i])
 	    {
@@ -663,7 +664,8 @@ void
 fillblock_rpm(struct blk *b, int id, struct seqdescr *sdesc, int nsdesc, struct fileblock *fb, int idx)
 {
   unsigned int size, nsize;
-  unsigned char *bp, *np;
+  unsigned char *bp;
+  char *np;
   int i, l, l2, u;
   struct seqdescr *sd;
   struct cpiophys cph;
@@ -696,7 +698,7 @@ fillblock_rpm(struct blk *b, int id, struct seqdescr *sdesc, int nsdesc, struct
 		l2 = l;
 	      if (S_ISLNK(fb->filemodes[sd->i]))
 		{
-		  strncpy(bp, symdata, l2);
+		  strncpy((char *)bp, symdata, l2);
 		  if (strlen(symdata) < l2)
 		    symdata += strlen(symdata);
 		  else
@@ -908,6 +910,8 @@ char *comp2str(int comp)
     return "bzip2";
   if (comp == CFILE_COMP_GZ)
     return "gzip";
+  if (comp == CFILE_COMP_GZ_RSYNC)
+    return "gzip rsyncable";
   if (comp == CFILE_COMP_UN)
     return "uncompressed";
   return "???";
@@ -1160,7 +1164,7 @@ main(int argc, char **argv)
 	  fprintf(stderr, "%s: bad payformat offset\n", deltarpm);
 	  exit(1);
 	}
-      strncpy(dh->dp + payformatoff, "cpio", 4);
+      strncpy((char *)dh->dp + payformatoff, "cpio", 4);
       inn = bzread4(bfp);
       outn = bzread4(bfp);
       in = xmalloc(2 * inn * sizeof(unsigned int));
@@ -1258,7 +1262,7 @@ main(int argc, char **argv)
 	printf("%02x", seq[i]);
       putchar('\n');
       printf("source rpm: %s\n", nevr);
-      printf("souce payload size: %u\n", outlen);
+      printf("source payload size: %u\n", outlen);
       printf("target rpm: %s\n", dhnevr);
       printf("target payload size: %u\n", paylen);
       if (targetcomp != CFILE_COMP_XX)
@@ -1446,7 +1450,7 @@ main(int argc, char **argv)
 	  fprintf(stderr, "addbz2: BZ2_bzDecompressInit error\n");
 	  exit(1);
 	}
-      addbz2strm.next_in = addbz2;
+      addbz2strm.next_in = (char *)addbz2;
       addbz2strm.avail_in = addbz2len;
       addbz2blk = xmalloc(BLKSIZE);
     }
@@ -1492,7 +1496,7 @@ main(int argc, char **argv)
 	      b = lastblk->e.buf + (off & BLKMASK);
 	      if (addbz2len)
 	        {
-		  addbz2strm.next_out = addbz2blk;
+		  addbz2strm.next_out = (char *)addbz2blk;
 		  addbz2strm.avail_out = l;
 		  BZ2_bzDecompress(&addbz2strm);
 		  if (addbz2strm.avail_out != 0)
