@@ -8,6 +8,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include <zlib.h>
 #include <bzlib.h>
@@ -26,14 +27,17 @@ cfile_readbuf(struct cfile *f, unsigned char *buf, int len)
   if (f->len != CFILE_LEN_UNLIMITED && len > f->len)
     len = f->len;
   if (!len)
-    return 0;
+    {
+      f->bufN = 0;
+      return 0;
+    }
 
   if (f->nunread)
     {
       int ur = len >= f->nunread ? f->nunread : len;
-      memcpy(buf, f->buf, ur);
+      memcpy(buf, f->unreadbuf, ur);
       if (f->nunread > ur)
-	memmove(f->buf + ur, f->buf, f->nunread - ur);
+	memmove(f->unreadbuf, f->unreadbuf + ur, f->nunread - ur);
       f->nunread -= ur;
       if (f->nunread == 0)
 	{
@@ -215,7 +219,7 @@ crread_bz(struct cfile *f, void *buf, int len)
 	  if (cfile_readbuf(f, f->buf, sizeof(f->buf)) == -1)
 	    return -1;
           f->strm.bz.avail_in = f->bufN;
-          f->strm.bz.next_in = f->buf;
+          f->strm.bz.next_in = (char *)f->buf;
         }
       used = f->strm.bz.avail_in;
       ret = BZ2_bzDecompress(&f->strm.bz);
@@ -223,7 +227,7 @@ crread_bz(struct cfile *f, void *buf, int len)
         return -1;
       used -= f->strm.bz.avail_in;
       if (used && f->ctxup)
-	f->ctxup(f->ctx, f->strm.bz.next_in - used, used);
+	f->ctxup(f->ctx, (unsigned char *)(f->strm.bz.next_in - used), used);
       f->bytes += used;
       if (ret == BZ_STREAM_END)
         {
@@ -265,7 +269,7 @@ cropen_bz(struct cfile *f)
     }
   f->eof = 0;
   f->strm.bz.avail_in = f->bufN == -1 ? 0 : f->bufN;
-  f->strm.bz.next_in  = f->buf;
+  f->strm.bz.next_in  = (char *)f->buf;
   return f;
 }
 
@@ -281,7 +285,7 @@ cwwrite_bz(struct cfile *f, void *buf, int len)
   for (;;)
     {
       f->strm.bz.avail_out = sizeof(f->buf);
-      f->strm.bz.next_out = f->buf;
+      f->strm.bz.next_out = (char *)f->buf;
       ret = BZ2_bzCompress(&f->strm.bz, BZ_RUN);
       if (ret != BZ_RUN_OK)
 	return -1;
@@ -303,7 +307,7 @@ cwclose_bz(struct cfile *f)
   for (;;)
     {
       f->strm.bz.avail_out = sizeof(f->buf);
-      f->strm.bz.next_out = f->buf;
+      f->strm.bz.next_out = (char *)f->buf;
       ret = BZ2_bzCompress(&f->strm.bz, BZ_FINISH);
       if (ret != BZ_FINISH_OK && ret != BZ_STREAM_END)
 	return -1;
@@ -568,7 +572,7 @@ cwopen_gz(struct cfile *f)
 
   f->crc = crc32(0L, Z_NULL, 0);
   f->crclen = 0;
-  ret = deflateInit2(&f->strm.gz, Z_BEST_COMPRESSION, Z_DEFLATED, -MAX_WBITS, 8, Z_DEFAULT_STRATEGY);
+  ret = deflateInit2(&f->strm.gz, Z_BEST_COMPRESSION, Z_DEFLATED, -MAX_WBITS, 8, Z_DEFAULT_STRATEGY | (f->comp == CFILE_COMP_GZ_RSYNC ? Z_RSYNCABLE : 0));
   if (ret != Z_OK)
     {
       free(f);
@@ -591,6 +595,7 @@ cwopen_gz(struct cfile *f)
   return f;
 }
 
+
 static int
 crunread_gz(struct cfile *f, void *buf, int len)
 {
@@ -663,6 +668,146 @@ crunread_un(struct cfile *f, void *buf, int len)
   return cfile_unreadbuf(f, buf, len, 1);
 }
 
+
+int
+cfile_detect_rsync(struct cfile *f)
+{
+  unsigned char *b, *b2;
+  int i, len, l, eof, p[2];
+  int comp = CFILE_COMP_GZ;
+  z_stream dstrm, cstrm[2];
+  int done, ret, dret;
+  unsigned char dbuf[4096], cbuf[4096];
+
+  if (f->comp != CFILE_COMP_GZ)
+    return 0;
+  b = malloc(4096 + f->strm.gz.avail_in);
+  if (!b)
+    return -1;
+  len = 0;
+
+  p[0] = p[1] = 0;
+
+  dstrm.zalloc = 0;
+  dstrm.zfree = 0;
+  dstrm.opaque = 0;
+  if (inflateInit2(&dstrm, -MAX_WBITS) != Z_OK)
+    {
+      free(b);
+      return -1;
+    }
+  for (i = 0; i < 2; i++)
+    {
+      cstrm[i].zalloc = 0;
+      cstrm[i].zfree = 0;
+      cstrm[i].opaque = 0;
+      if (deflateInit2(&cstrm[i], Z_BEST_COMPRESSION, Z_DEFLATED, -MAX_WBITS, 8, Z_DEFAULT_STRATEGY | (i == 1 ? Z_RSYNCABLE : 0)) != Z_OK)
+	{
+	  if (i)
+	    deflateEnd(&cstrm[0]);
+	  inflateEnd(&dstrm);
+	  free(b);
+          return -1;
+	}
+    }
+
+  done = eof = 0;
+  dstrm.avail_in = f->strm.gz.avail_in;
+  if (f->strm.gz.avail_in)
+    memcpy(b, f->strm.gz.next_in, f->strm.gz.avail_in);
+  for (;;)
+    {
+      if (dstrm.avail_in == 0)
+	{
+          l = cfile_readbuf(f, b + len, 4096);
+	  if (l < 4096)
+	    eof = 1;
+	}
+      else
+	l = dstrm.avail_in;
+      if (l >= 0)
+	{
+	  dstrm.avail_in = l;
+	  dstrm.next_in = b + len;
+	  while (dstrm.avail_in && !done)
+	    {
+	      dstrm.avail_out = sizeof(dbuf);
+	      dstrm.next_out = dbuf;
+	      dret = inflate(&dstrm, Z_NO_FLUSH);
+	      if (dret != Z_OK && dret != Z_STREAM_END)
+		{
+		  done = 1;
+		  break;
+		}
+	      if (dstrm.avail_out != sizeof(dbuf))
+		{
+		  for (i = 0; i < 2 && !done; i++)
+		    {
+		      cstrm[i].avail_in = sizeof(dbuf) - dstrm.avail_out;
+		      cstrm[i].next_in = dbuf;
+		      while (cstrm[i].avail_in)
+			{
+			  cstrm[i].avail_out = sizeof(cbuf);
+			  cstrm[i].next_out = cbuf;
+			  ret = deflate(&cstrm[i], dret == Z_STREAM_END ? Z_FINISH : Z_NO_FLUSH);
+			  if (ret != Z_OK && ret != Z_STREAM_END)
+			    {
+			      comp = i ? CFILE_COMP_GZ: CFILE_COMP_GZ_RSYNC;
+			      done = 1;
+			      break;
+			    }
+			  if (cstrm[i].avail_out != sizeof(cbuf))
+			    {
+			      if (memcmp(b + p[i], cbuf, sizeof(cbuf) - cstrm[i].avail_out))
+				{
+				  comp = i ? CFILE_COMP_GZ: CFILE_COMP_GZ_RSYNC;
+				  done = 1;
+				  break;
+				}
+			      p[i] += sizeof(cbuf) - cstrm[i].avail_out;
+			    }
+			  if (cstrm[i].avail_in && ret == BZ_STREAM_END)
+			    {
+			      comp = i ? CFILE_COMP_GZ: CFILE_COMP_GZ_RSYNC;
+			      break;
+			    }
+			}
+		    }
+		}
+	      if (dret == Z_STREAM_END)
+		done = 1;
+	    }
+	  len += l;
+	}
+      if (done || eof)
+	break;
+      b2 = realloc(b, len + 4096);
+      if (!b2)
+	{
+	  comp = -1;
+	  break;
+	}
+      b = b2;
+    }
+  deflateEnd(&cstrm[0]);
+  deflateEnd(&cstrm[1]);
+  inflateEnd(&dstrm);
+  f->bufN = -1;
+  f->strm.gz.avail_in = 0;
+  if (comp != -1)
+    f->comp = comp;
+  if (len)
+    {
+      if (f->len != CFILE_LEN_UNLIMITED)
+	f->len += len;
+      f->nunread = len;
+      f->unreadbuf = b;
+    }
+  else
+    free(b);
+  return comp == -1 ? -1 : 0;
+}
+
 /*****************************************************************
  *  our open function
  */
@@ -719,6 +864,7 @@ cfile_open(int mode, int fd, void *fp, int comp, unsigned int len, void (*ctxup)
       f->close  = mode == CFILE_OPEN_RD ? crclose_un : cwclose_un;
       return mode == CFILE_OPEN_RD ? cropen_un(f) : cwopen_un(f);
     case CFILE_COMP_GZ:
+    case CFILE_COMP_GZ_RSYNC:
       f->strm.gz.zalloc = 0;
       f->strm.gz.zfree = 0;
       f->strm.gz.opaque = 0;
