@@ -70,7 +70,7 @@ static int blockwrite(struct bzblock *bzf, void *buf, int len)
 	    return -1;
 	  bzf->strm.avail_out = 8192;
 	}
-      bzf->strm.next_out = bzf->data + bzf->len;
+      bzf->strm.next_out = (char *)bzf->data + bzf->len;
       ret = BZ2_bzCompress(&bzf->strm, BZ_RUN);
       if (ret != BZ_RUN_OK)
 	return -1;
@@ -97,7 +97,7 @@ static int blockclose(struct bzblock *bzf, unsigned char **datap, int *lenp)
 	    return -1;
 	  bzf->strm.avail_out = 8192;
 	}
-      bzf->strm.next_out = bzf->data + bzf->len;
+      bzf->strm.next_out = (char *)bzf->data + bzf->len;
       ret = BZ2_bzCompress(&bzf->strm, BZ_FINISH);
       if (ret != BZ_FINISH_OK && ret != BZ_STREAM_END)
         return -1;
