@@ -491,7 +491,7 @@ put32(FILE *fp, unsigned int i)
   fputc(i      , fp);
 }
 
-void addtocpio(unsigned char **cpiop, int *cpiol, unsigned char *d, int l)
+void addtocpio(unsigned char **cpiop, unsigned int *cpiol, unsigned char *d, int l)
 {
   int cpl = *cpiol;
   if ((cpl & ~4095) == 0 || (cpl & ~4095) + l > 4096)
@@ -544,7 +544,7 @@ main(int argc, char **argv)
   unsigned char *newcpio = 0;
   unsigned int newcpiolen = 0;
 
-  unsigned char *payformat;
+  char *payformat;
   MD5_CTX fullmd5;
   unsigned char fullmd5res[16];
   unsigned int fullsize;
@@ -706,7 +706,7 @@ main(int argc, char **argv)
     }
   fileflags = headint32(h, TAG_FILEFLAGS, (int *)0);
   filemd5s = headstringarray(h, TAG_FILEMD5S, (int *)0);
-  filerdevs = headint32(h, TAG_FILERDEVS, (int *)0);
+  filerdevs = headint16(h, TAG_FILERDEVS, (int *)0);
   filesizes = headint32(h, TAG_FILESIZES, (int *)0);
   filemodes = headint16(h, TAG_FILEMODES, (int *)0);
   fileverify = headint32(h, TAG_FILEVERIFY, (int *)0);
@@ -815,25 +815,25 @@ main(int argc, char **argv)
 	}
       if (!skip)
 	{
-	  char cpiobuf[110 + 3];
+	  unsigned char cpiobuf[110 + 3];
 	  int ns = strlen(np);
-	  sprintf(cpiobuf, "07070100000000%08x00000000000000000000000100000000%08x0000000000000000%08x%08x%08x00000000./", filemodes[i], lsize, devmajor(rdev), devminor(rdev), ns + 3);
+	  sprintf((char *)cpiobuf, "07070100000000%08x00000000000000000000000100000000%08x0000000000000000%08x%08x%08x00000000./", filemodes[i], lsize, devmajor(rdev), devminor(rdev), ns + 3);
 	  addtocpio(&oldcpio, &oldcpiolen, cpiobuf, 112);
-	  addtocpio(&oldcpio, &oldcpiolen, np, ns + 1);
+	  addtocpio(&oldcpio, &oldcpiolen, (unsigned char *)np, ns + 1);
 	  ns += 3 - 2;
 	  for (; ns & 3 ; ns++)
-	    addtocpio(&oldcpio, &oldcpiolen, "", 1);
-	  rpmMD5Update(&seqmd5, np, strlen(np) + 1);
+	    addtocpio(&oldcpio, &oldcpiolen, (unsigned char *)"", 1);
+	  rpmMD5Update(&seqmd5, (unsigned char *)np, strlen(np) + 1);
 	  rpmMD5Update32(&seqmd5, filemodes[i]);
 	  rpmMD5Update32(&seqmd5, lsize);
 	  rpmMD5Update32(&seqmd5, rdev);
 	  if (S_ISLNK(filemodes[i]))
 	    {
-	      addtocpio(&oldcpio, &oldcpiolen, filelinktos[i], lsize);
+	      addtocpio(&oldcpio, &oldcpiolen, (unsigned char *)filelinktos[i], lsize);
 	      for (; lsize & 3 ; lsize++)
-	        addtocpio(&oldcpio, &oldcpiolen, "", 1);
+	        addtocpio(&oldcpio, &oldcpiolen, (unsigned char *)"", 1);
 	      skip = 1;
-	      rpmMD5Update(&seqmd5, filelinktos[i], strlen(filelinktos[i]) + 1);
+	      rpmMD5Update(&seqmd5, (unsigned char *)filelinktos[i], strlen(filelinktos[i]) + 1);
 	    }
 	  if (S_ISREG(filemodes[i]) && lsize)
 	    {
@@ -855,7 +855,7 @@ main(int argc, char **argv)
 	      exit(1);
 	    }
 	  if (!skip)
-	    addtocpio(&oldcpio, &oldcpiolen, buf, l3);
+	    addtocpio(&oldcpio, &oldcpiolen, (unsigned char *)buf, l3);
 	  l -= l3;
 	}
       if ((l2 & 3) != 0)
@@ -867,13 +867,13 @@ main(int argc, char **argv)
 	      exit(1);
 	    }
 	  if (!skip)
-	    addtocpio(&oldcpio, &oldcpiolen, "\0\0\0", l2);
+	    addtocpio(&oldcpio, &oldcpiolen, (unsigned char *)"\0\0\0", l2);
 	}
     }
   namebuf = xfree(namebuf);
   namebufl = 0;
   bfd->close(bfd);
-  addtocpio(&oldcpio, &oldcpiolen, "07070100000000000000000000000000000000000000010000000000000000000000000000000000000000000000000000000b00000000TRAILER!!!\0\0\0\0", 124);
+  addtocpio(&oldcpio, &oldcpiolen, (unsigned char *)"07070100000000000000000000000000000000000000010000000000000000000000000000000000000000000000000000000b00000000TRAILER!!!\0\0\0\0", 124);
   close(fd);
 
  
@@ -962,9 +962,14 @@ main(int argc, char **argv)
       fprintf(stderr, "payload open failed\n");
       exit(1);
     }
+  if (cfile_detect_rsync(newbz))
+    {
+      fprintf(stderr, "detect_rsync failed\n");
+      exit(1);
+    }
   comp = newbz->comp;
   while ((l = newbz->read(newbz, buf, sizeof(buf))) > 0)
-    addtocpio(&newcpio, &newcpiolen, buf, l);
+    addtocpio(&newcpio, &newcpiolen, (unsigned char *)buf, l);
   if (l < 0)
     {
       fprintf(stderr, "payload read failed\n");
